package org.apache.maven.plugin.war.stub;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.HashSet;
import java.util.Set;

import org.apache.maven.artifact.Artifact;

/**
 * @author <a href="mailto:olamy@apache.org">olamy</a>
 * @since 9 juin 07
 * @version $Id: MavenZipProject.java 985726 2010-08-15 18:13:26Z bentmann $
 */
public class MavenZipProject
    extends MavenProject4CopyConstructor
{
    private Set artifacts;

    public MavenZipProject()
        throws Exception
    {
        super();
        this.artifacts = new HashSet();
    }

    public Set getArtifacts()
    {
        return this.artifacts;
    }
    
    public Set getDependencyArtifacts()
    {
        return this.artifacts;
    }

    public void addArtifact( Artifact artifact )
    {
        this.artifacts.add( artifact );
    }

}
